#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <mcheck.h>


/* #define NDEBUG */


/*
 * Dijkstra's algorithm for calculating minimum paths from the given vertex to
 * all other vertices of the graph.  Weights are non-negative.  -1 means
 * infinite weight (no path)
 */



/* Item of a path list */
typedef struct _Pitem {
    int vertex;
    struct _Pitem *next;
} Pitem;

/* Item of the D or S sets (sets of unprocessed and processed vertices) */
typedef struct _Ditem {
    int vertex;
    int min_path;
    struct _Ditem *next;
} Ditem;



char *progname = NULL;
void dijkstra( int num_vertices, int *graph, int source,
               Pitem ***paths_ret, Ditem **path_vals_ret );



/* Stay out of the way of memory allocation tracing in debugging mode */
#ifndef NDEBUG
#define xmalloc(size)       malloc(size)
#else /* NDEBUG */
void *xmalloc(int size)
{
    void *ptr = malloc(size);
    if ( !ptr ) {
        fprintf(stderr, "%s: cannot allocate memory\n", progname);
        exit(1);
    }
    return ptr;
}
#endif /* NDEBUG */



int main(int argc, char **argv)
{
    int *graph        = NULL;
    int  num_vertices = 0;
    int  source;
    int  i, j;
    Ditem *path_vals  = NULL;
    Pitem **paths     = NULL;

#ifndef NDEBUG
    mtrace();
#endif /* NDEBUG */

    progname = *argv;
    /*
     * Input the graph
     */
    /* Read in number of vertices in the graph and source vertex */
    if ( 2 != scanf("%d %d", &num_vertices, &source) ) {
        fprintf(stderr, "%s: unexpected end of file\n", progname);
        exit(1);
    }
    if ( num_vertices <= 0 ) {
        fprintf(stderr,
                "%s: error--number of vertices should be a positive integer\n",
                progname);
        exit(1);
    }
    if ( source < 0 || source >= num_vertices ) {
        fprintf(stderr, "%s: error--invalid source vertex\n", progname);
        exit(1);
    }
    /* Allocate space for and read in incidence matrix for the graph */
    j = num_vertices*num_vertices;
    graph = (int *) xmalloc(j * sizeof(int));
    for ( i = 0; i < j; i++ )
        if ( 1 != scanf("%d", graph + i) ) {
            fprintf(stderr, "%s: unexpected end of file\n", progname);
            exit(1);
        }
    /*
     * Do the job
     */
    dijkstra(num_vertices, graph, source, &paths, &path_vals);
    /*
     * Print out the graph
     */
    printf("%d %d\n\n", num_vertices, source);
    for ( i = 0; i < num_vertices; i++ ) {
        for ( j = 0; j < num_vertices; j++ )
            printf("%d ", *(graph + i*num_vertices + j));
        putchar('\n');
    }
    /*
     * Print out minimal path weights
     */
    putchar('\n');
    while ( path_vals ) {
        if ( path_vals->min_path < 0 )
            printf("%d\t$\\infty$\n", path_vals->vertex);
        else
            printf("%d\t%d\n", path_vals->vertex, path_vals->min_path);
        path_vals = path_vals->next;
    }
    /*
     * Print out the paths (source and destination vertices are not in the lists)
     */
    putchar('\n');
    for ( i = 0; i < num_vertices; i++ ) {
        Pitem *curr = *(paths + i);

        if ( i == source )
            continue;

        printf("%d ", source);
        while ( curr ) {
            printf("%d ", curr->vertex);
            curr = curr->next;
        }
        puts("-1"); /* path terminator */
    }

    return 0;
}



void path_free(Pitem *path)
{
    Pitem *next;

    while (path) {
        next = path->next;
        free(path);
        path = next;
    }

    return;
}

Pitem *path_copy(Pitem *path, Pitem **tail_ret)
{
    if ( path ) {
        Pitem *head, *tail;

        head = xmalloc(sizeof(Pitem));
        head->vertex = path->vertex;
        head->next   = NULL;
        tail = head;
        path = path->next;
        while ( path ) {
            tail = tail->next = (Pitem *) xmalloc(sizeof(Pitem));
            tail->vertex = path->vertex;
            tail->next   = NULL;
            path         = path->next;
        }

        if ( tail_ret )
            *tail_ret = tail;
        return head;
    }

    if ( tail_ret )
        *tail_ret = NULL;
    return NULL;
}

Pitem *path_add_vertex(Pitem *head, int vertex)
{
    Pitem *tail;

    if ( !head )
        head = tail = (Pitem *) xmalloc(sizeof(Pitem));
    else {
        tail = head;
        while ( tail->next )
            tail = tail->next;
        tail = tail->next = (Pitem *) xmalloc(sizeof(Pitem));
    }

    tail->vertex = vertex;
    tail->next   = NULL;

    return head;
}



#define WEIGHT(i, j)        ( *(graph + (i) * num_vertices + (j)) )

void dijkstra( int num_vertices, int *graph, int source,
               Pitem ***paths_ret, Ditem **path_vals_ret )
{
    int i;
    Ditem *D      = NULL;   /* list of unprocessed vertices */
    Ditem *S      = NULL;   /* list of processed vertices */
    Pitem **paths = NULL;   /* array of path lists for each vertex (excluding
                             * source and destination vertices) */

    assert(path_vals_ret || paths_ret);

    /*
     * Initialization
     */

    /* Allocate and initialize linked list for the D set (set of unprocessed
     * vertices) */
    for ( i = 0; i < num_vertices; i++ )
        if ( i != source ) {
            Ditem *ptr  = (Ditem *) xmalloc(sizeof(Ditem));
            ptr->next   = D;
            D           = ptr;
            D->vertex   = i;
            D->min_path = WEIGHT(source, i);
        }
    /* Allocate and initialize one item for the S set (set of processed
     * vertices) */
    S           = (Ditem *) xmalloc(sizeof(Ditem));
    S->next     = NULL;
    S->vertex   = source;
    S->min_path = 0;
    /* Allocate array of num_vertices pointers to path lists' heads */
    paths = (Pitem **) xmalloc(sizeof(Pitem*)*num_vertices);
    memset(paths, 0, sizeof(Pitem*)*num_vertices);

    /*
     * The algorithm proper
     */

    /* Loop while there are unprocessed vertices left */
    while ( D ) {
        int    min;
        Ditem *curr, *prev, *ptr_min;
        /* Among the unprocessed vertices, select for processing the one with
         * the lowest min_path value */
        min     = D->min_path;
        ptr_min = NULL; /* this will be vertex previous to the one we find */
        curr    = D->next;
        prev    = D;
        while ( curr ) {
            if ( curr->min_path >= 0 && /* not infinity */
                 (min > curr->min_path || min < 0) ) {
                min     = curr->min_path;
                ptr_min = prev;
            }
            prev = curr;
            curr = curr->next;
        }
        /* Unlink the vertex we've found from the D set */
        if ( !ptr_min ) {   /* vertex we've found is the head of D list */
            ptr_min = D;
            D       = D->next;
        } else {            /* vertex we've found is not the head of D list */
            prev          = ptr_min->next;
            assert(ptr_min->next);
            ptr_min->next = ptr_min->next->next;
            ptr_min       = prev;
        }
        assert(ptr_min);
        /* Link the vertex into the S set */
        ptr_min->next = S;
        S             = ptr_min;
        if ( S->min_path >= 0 ) { /* take trouble only if reachable */
            /* Add ptr_min vertex to the end of the path */
            *(paths + S->vertex) = path_add_vertex(*(paths + S->vertex), S->vertex);
            /* Recalculate all min_path's for each remaining vertex in the D set and
             * update paths of those which have a better path from the source vertex
             * running thru ptr_min vertex */
            curr = D;
            while ( curr ) {
                int new = WEIGHT(S->vertex, curr->vertex) + S->min_path;
                if ( WEIGHT(S->vertex, curr->vertex) >= 0 && /* not infinity */
                     (new < curr->min_path || curr->min_path < 0) ) {
                    Pitem *tail;
                    /* Update path's weight */
                    curr->min_path = new;
                    /* Free up old path */
                    path_free(*(paths + curr->vertex));
                    /* Copy the path from source to ptr_min vertex */
                    *(paths + curr->vertex) = path_copy(*(paths + S->vertex), &tail);
                }
                curr = curr->next;
            }
        }
    }

    /* Return S or free it up */
    if ( path_vals_ret )
        *path_vals_ret = S;
    else
        while ( S ) {
            Ditem *next = S->next;
            free(S);
            S = next;
        }
    /* Return paths or free it up */
    if ( paths_ret )
        *paths_ret = paths;
    else {
        for ( i = 0; i < num_vertices; i++ )
            path_free(*(paths + i));
        free(paths);
    }

    return;
}



/*
 * Local variables:
 * compile-command:  "make -k dijkstra && MALLOC_TRACE=malloc.trace ./dijkstra < graph1"
 * End:
 */
