#!/usr/bin/python

import types
import getopt
from sys import *
from math import sqrt, sin, cos, fabs, pi, ceil
from time import strftime

#
# Skips all white-space characters and reads in following non-white-space
# characters.  Returns the string (empty if no more input available)
#
def scantoken(file=stdin):
    c = file.read(1)
    while c != "" and c.isspace():
        c = file.read(1)
    str = ""
    while c != "" and not c.isspace():
        str = str + c
        c = file.read(1)
    return str

#
# Format string must be a white-space delimited sequence of conversion
# specifiers.  No width or precision modifiers are supported.
#
# The following conversions are supported:
#    %d, %u         - decimal integer
#    %x, %X         - hexademical integer
#    %o             - octal integer
#    %f, %e, %E, %g - floating point number
#    %s             - sequence of non-white-space characters
#
# Return value is a list of values parsed according to the format string
#
# Exceptions raised:
#    TypeError      - fstring is not of type `string'
#    IndexError     - EOF before all of the format string have been satisfied
#    ValueError     - illegal conversion specifier in format string
#
# Note that ValueError is also raised by string-to-numeric conversion functions
# on illegal input
#
def scanf(fstring, file=stdin):
    if type(fstring) != types.StringType:
        raise TypeError, "illegal type of format string argument"
    result = []
    flist = fstring.split()
    for format in flist:
        str = scantoken(file)
        if str == "":
            raise IndexError, "unexpected EOF--not all arguments read for format string"
        if format == "%d" or format == "%u":
            result = result + [int(str)]
        elif format == "%x" or format == "%X":
            result = result + [int(str, 16)]
        elif format == "%o":
            result = result + [int(str, 8)]
        elif format == "%f" or format == "%e" or format == "%g" or format == "%E":
            result = result + [float(str)]
        elif format == "%s":
            result = result + [str]
        else:
            raise ValueError, "illegal type modifier in format string"
    return tuple(result)



# Returns a string (one of "lft", "rt", "top", "bot", "ulft", "urt", "llft",
# and "lrt") which is a label suffix which should be used for label on an edge
# starting at `fr' point and ending at 'to' point.  `fr' and `to' arguments
# must each be a sequence of two numbers (x and y coordinates)
def edge_label_suffix(fr, to):
    cos_     = (to[0]-fr[0]) / sqrt( (to[0]-fr[0])**2 + (to[1]-fr[1])**2 )
    if to[1]-fr[1] > 0: sin_sgn = 1
    else: sin_sgn = -1

    if fabs(cos_) > cos(pi/8):   # "clinging" to Ox
        if cos_ > 0:    # east
            return "top"
        else:           # west
            return "bot"
    elif fabs(cos_) < sin(pi/8): # "clinging" to Oy
        if sin_sgn > 0: # north
            return "lft"
        else:           # south
            return "rt"
    elif cos_ * sin_sgn > 0:           # "clinging" to y=x
        if cos_ > 0:    # east
            return "ulft"
        else:           # west
            return "lrt"
    else:                       # "clinging" to y=-x
        if cos_ > 0:    # east
            return "urt"
        else:           # west
            return "llft"

# Returns a string (one of "lft", "rt", "top", "bot", "ulft", "urt", "llft",
# and "lrt") which is a label suffix to be used for label on a vertex
# whose position in graph's table is given by `pos' argument (sequence of two
# numbers).  Dimensions of the table are `width' by 'height'
def vertex_label_suffix(pos, width, height):
    if pos[0] <= 0:
        if pos[1] <= 0:
            return "llft"
        elif pos[1] >= height - 1:
            return "ulft"
        else:
            return "lft"
    elif pos[0] >= width - 1:
        if pos[1] <= 0:
            return "lrt"
        elif pos[1] >= height - 1:
            return "urt"
        else:
            return "rt"
    else:
        if pos[1] <= 0:
            return "bot"
        else:
            return "top"



def output_header(argv):
    #
    # Timestamp and creator
    #
    cmdline = ""
    for arg in argv:
        cmdline = cmdline + " " + arg
    print """%
% Generated with `""" + cmdline + """'
% on """ + strftime("%a %b %d %H:%M:%S %Z %Y") + """
%"""
    #
    # `verbatimtex' header
    #
    print """verbatimtex %&cyrtex
\\fontencoding{T2A}%
\\font\eightrm = wcmr8
\\font\sevenrm = wcmr7
\\font\sixrm = wcmr6
\\inputencoding{koi8-r}%
\\Russian % select russian (for hyphenation)
\\tenrm
etex


"""



def output_data_construction(graph):
    global unit, skew_out, skew_in, vert_spacing, vert_size

    # Number of vertices in the graph
    num_vertices = len(graph)
    # Width of the graph's table
    n = int(ceil(sqrt(num_vertices)))
    #if n != int(n): n = int(n) + 1
    # Height of the graph's table
    m = int((num_vertices + n - 1)/n)

    #
    # Variable declarations and parameters
    #
    print """numeric vcount, vsize, vspc, so, si, u;
numeric M[]x[], I[]x[], O[]x[];
path    V[], E[]x[];
picture Elab[]x[], Vlab[];
pair    vert[];

numeric i, j;
pair    outdir, indir;

so       = """ + `skew_out` + """; % Skew angle for out-going edges
si       = """ + `skew_in`  + """; % Skew angle for in-going edges
vcount   = """ + `num_vertices` + """;
u        = """ +  unit + """;
vsize    = """ + `vert_size` + """u;
vspc     = """ + `vert_spacing` + "u;"
    #
    # List vertexes and their coordinates
    #
    print "\n% Coordinates of the vertices"
    z = []
    for i in range(m):
        for j in range(n):
            if i * n + j >= num_vertices: break
            stdout.write("vert" + `i*n+j` + " = (" + \
                         `j+1` + "*vspc," + \
                         `m-i` + "*vspc); ")
            z = z + [[j, m-i-1]]
        print
    #
    # Print out a loop which constructs paths for each vertex
    #
    print "\n% Paths for the vertices"
    print """for i := 0 upto vcount-1:
  V[i] = fullcircle scaled vsize shifted vert[i];
endfor;"""
    #
    # List labels for all vertices
    #
    print "\n% Labels for the vertices"
    print "begingroup"
    print "  save labeloffset;"
    print "  labeloffset := vsize/2*1.25;"
    for i in range(num_vertices):
        print "  Vlab[" + `i` + "] := thelabel." + vertex_label_suffix(z[i], n, m) + \
              "(btex \it " + `i` + " etex, vert[" + `i` + "]);"
    print "endgroup;"
    #
    # List elements of the incidence matrix
    #
    print "\n% Incidence matrix"
    for i in range(num_vertices):
        for j in range(num_vertices):
            stdout.write("M" + `i` + "x" + `j` + ":=" + `graph[i][j]` + "; ")
        print
    #
    # Out-going and in-going skew angles for edges
    #
    print "\n% Out-going and in-going skew angles for the edges"
    for i in range(num_vertices):
        for j in range(num_vertices):
            stdout.write("O[" + `i` + "]x[" + `j` + "]:=so;I[" + `i` + "]x[" + `j` + "]:=si; ")
        print
    #
    # Print out a loop which will construct paths for all edges
    #
    print "\n% Construct paths for the edges"
    print """for i := 0 upto vcount-1:
  for j := 0 upto vcount-1:
    if M[i]x[j] >= 0:
      outdir   := direction 0 of (vert[i]--vert[j] rotatedaround(vert[i], O[i]x[j]));
      indir    := direction 0 of (vert[i]--vert[j] rotatedaround(vert[i], I[i]x[j]));
      E[i]x[j] := vert[i]{outdir}..{indir}vert[j] cutbefore V[i] cutafter V[j];
    fi;
  endfor;
endfor;
"""
    #
    # List labels for all edges
    #
    print "% Labels for the edges"
    for i in range(num_vertices):
        for j in range(num_vertices):
            if graph[i][j] >= 0:
                print "Elab[" + `i` + "]x[" + `j` + "] := thelabel." + \
                      edge_label_suffix(z[i], z[j]) + "(btex " + `graph[i][j]` + " etex, " + \
                      "point .5 of E[" + `i` + "]x[" + `j` + "]);"
    print "\n\n"


def output_fig(fig_num, graph, marks, source, path = []):
    print "beginfig(" + `fig_num` + ");"
    #
    # Print out a loop which will draw path
    #
    if len(path) > 1:
        print "  % Draw the path"
        print "  pickup pencircle scaled 4pt;"
        for i in range(len(path)-1):
            print "  drawarrow E[" + `path[i]` + "]x[" + `path[i+1]` + "] withcolor .8white;"
        print "  pickup defaultpen;"
    #
    # Print out a loop which will draw edges with labels
    #
    print "  % Draw edges with labels"
    print """  for i := 0 upto vcount-1:
    for j := 0 upto vcount-1:
      if M[i]x[j] >= 0:
        drawarrow E[i]x[j];
        draw Elab[i]x[j];
      fi;
    endfor;
  endfor;"""
    #
    # Print out a loop which will draw vertices with labels
    #
    print "\n  % Draw vertices with labels"
    print """  for i := 0 upto vcount-1:
    if i = """ + `source` + """:
      filldraw V[i] withcolor .8white;
    else:
      filldraw V[i] withcolor white;
    fi;
    draw V[i];
    draw Vlab[i];
  endfor;"""
    #
    # Draw marks for all vertices
    #
    print "\n  % Marks for the vertices"
    for i in range(num_vertices):
        print "  label(btex " + marks[i] + " etex, vert[" + `i` + "]);"
    #
    # Wrap up
    #
    print "endfig;\n\n"



def output_closing():
    print "\nend"



#
# main()
#
errUnexpEOFnumSource = "Error--unexpected EOF (expected number of vertices and source vertex)\n"
errUnexpEOFmark      = "Error--unexpected EOF (expected vertex mark)\n"
errUnexpEOFgraph     = "Error--unexpected EOF (expected incidence matrix)\n"
errParsingNumSource  = "Error parsing number of vertices and source vertex\n"
errParsingGraph      = "Error parsing incidence matrix\n"
errParsingMark       = "Error parsing vertex mark\n"
errParsingPath       = "Error parsing path list\n"
errMarkVertex        = "Error--invalid vertex number in the marks list (%d)\n"
errMarkRedef         = "Error--mark for vertex `%d' already defined\n"
errFileOpen          = "Error--cannot open file `%s'\n"

# Default values for parameters
stream_in    = stdin
unit         = ".25cm"
vert_spacing = 10
vert_size    = 2
skew_out     = 15
skew_in      = -15

#
# Parse command line arguments
#
errTryHelp          = "Try `" + argv[0] + " -h' for help\n"
usage = """
Usage: """ + argv[0] + """ [-h] [-i fname] [-u unit] [-p spacing] [-s size]
            [-O skew_out] [-I skew_in]

Generates METAPOST code for drawing a graph and (optionally) graphs with
paths.

-h            show this help message
-i fname      file to read input from, `-' for stdin (default: stdin)
-u unit       basic unit for generated METAPOST code (default: `%s')
-p spacing    spacing to use for vertices            (default: %3d)
-s size       size to use for vertices               (default: %3d)
-O skew_out   degrees to deflect out-going arrows    (default: %3d)
-I skew_in    degrees to deflect in-going arrows     (default: %3d)

Note:  input should be a graph and any number of path lists presented as
       follows:
       - number of vertices in the graph and source vertex (will be shaded
         and used as a starting vertex for path lists, see below);
       - incidence matrix, with non-negative weights of edges (infinite
         weight, i.e. lack of an edge, must be specified by a negative
         weight);
       - `vertex number' / `vertex label' pair for each vertex, where
         `vertex label' can be any sequence of non-white-space characters;
       - optionally, any number of paths (starting at source vertex).  For
         each of the paths, the program will output code for drawing one
         additional graph  with the path highlighted.  A path is a list of
         vertices (the source vertex is implied as start of the path, so do
         not specify it), with the end of the list marked by a negative
         integer
"""

try:
    optionlist, args = getopt.getopt(argv[1:], "i:u:p:s:O:I:h")
except getopt.GetoptError, (msg, opt):
    stderr.write(msg + "\n")
    stderr.write(errTryHelp)
    exit(1)

for opt, val in optionlist:
    try:
        if opt == "-i":
            if val != "-":
                stream_in = open(val, "r")
    except (IOError):
        stderr.write(errFileOpen % val)
        exit(1)

    if opt == "-u":
        unit = val

    try:
        if opt == "-p":
            vert_spacing = int(val)
        if opt == "-s":
            vert_size = int(val)
        if opt == "-O":
            skew_out = int(val)
        if opt == "-I":
            skew_in = int(val)
    except (ValueError, SyntaxError, NameError):
        stderr.write(errIntegerReq % (opt))
        stderr.write(errTryHelp)
        exit(1)

    if opt == "-h":
        print usage % (unit, vert_spacing, vert_size, skew_out, skew_in)
        exit(0)
#
# Number of vertices and source vertex
#
try:
    num_vertices, source = scanf("%d %d", stream_in)
except IndexError:
    stderr.write(errUnexpEOFnumSource); exit(1)
except ValueError:
    stderr.write(errParsingNumSource); exit(1)

if num_vertices <= 0:
    stderr.write("Error--number of vertices must be a positive integer (%d given)\n" % num_vertices)
    exit(1)

if source < 0 or source >= num_vertices:
    stderr.write("Error--invalid source vertex number given (%d)\n" % source)
    exit(1)
#
# Read in the graph
#
graph = []
for i in range(num_vertices):
    graph = graph + [[]]
    for j in range(num_vertices):
        try:
            t, = scanf("%d", stream_in)
        except IndexError:
            stderr.write(errUnexpEOFgraph); exit(1)
        except ValueError:
            stderr.write(errParsingGraph); exit(1)
        else:
            if t < 0: t = -1
            graph[i] = graph[i] + [t]
#
# Read in vertex marks
#
marks = []
for i in range(num_vertices):
    marks = marks + [""]
for i in range(num_vertices):
    try:               vert, mark = scanf("%d %s", stream_in)
    except IndexError: stderr.write(errUnexpEOFmark); exit(1)
    except ValueError: stderr.write(errParsingMark); exit(1)
    if vert < 0 or vert >= num_vertices: stderr.write(errMarkVertex % vert); exit(1)
    if marks[vert] != "": stderr.write(errMarkRedef % vert); exit(1)
    marks[vert] = mark
#
# Read in paths
#
finished = 0
i        = 0
vert     = 0
paths    = []

while not finished:
    # Read start of a path
    try:               vert, = scanf("%d", stream_in)
    except IndexError: break
    except ValueError: stderr.write(errParsingPath); exit(1)
    paths = paths + [[]]
    # Go on reading until negative vertex number encountered
    while vert >= 0:
        if vert >= num_vertices:
            stderr.write("Error--invalid vertex number in the path (%d)\n" % vert)
            exit(1)
        paths[i] = paths[i] + [vert]
        try:               vert, = scanf("%d", stream_in)
        except IndexError: finished = 1; break
        except ValueError: stderr.write(errParsingPath); exit(1)
    i = i + 1

#
# Output METAPOST code
#
output_header(argv)
output_data_construction(graph)

# A graph without paths
output_fig(1, graph, marks, source)
# One graph for each of the paths given
i = 2
for p in paths:
    output_fig(i, graph, marks, source, p)
    i = i + 1

output_closing()



#
# Local variables:
# compile-command:  "make -k graph2.pdf && gv graph2.pdf"
# End:
#
