/*
 *                         (H1,V1)
 *         an1 an2 ... ann
 *    ^
 *  H |    ... ... ... ...
 *    |
 *         a21 a22 ... a2n
 *         a11 a12 ... a1n
 * (H0,V0)
 *             --->
 *              V
 *
 * An airplane has to climb height from H0 to H1 and to gain velocity from V0 to
 * V1.  The above matrix represents values of fuel consumption required to climb
 * next height or to gain next velocity from each of the positions between
 * (H0,V0) and (H1, V1).  Calculate the smallest possible value of fuel
 * consumption required for the plane to achieve the goal.  The plane cannot
 * loose height or velocity (it can "travel" the matrix only up or right).
 */
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>


char *errUnexpectedEOF = "%s: error--unexpected end of file\n";
char *errParsingInput  = "%s: error--invalid input\n";
char *errMemory        = "%s: error--cannot allocate enough memory\n";

char *progname;

unsigned int  n, nn, n_minus_one;  /* dimension of HV-matrix */
unsigned int *a;                    /* HV-matrix */
unsigned int *m; /* 2-D array to hold intermediate calculations */


void calc_diagonal_on_and_above(int i, int j);
void calc_diagonal_below(int i, int j);
unsigned int read_unsigned_int(void);


#define M(i, j)         *(m + (i)*n   + j)
#define A(i, j)         *(a + (i)*n   + j)
#define M_up(i, j)      *(m + (i+1)*n + j)
#define M_rt(i, j)      *(m + (i)*n   + j+1)


int main(int argc, char **argv)
{
    int i, j;

    progname = *argv;

    /* Read matrix dimensions */
    n = read_unsigned_int();
    nn = n * n;
    n_minus_one = n - 1;

    /* Allocate buffer long enough to hold HV-matrix */
    a = (unsigned *) malloc( sizeof(int) * nn );
    /* Allocate 2-D buffer for (m * n) integers */
    m = (unsigned *) malloc( sizeof(int) * nn );
    if ( !a || !m ) {
        fprintf(stderr, errMemory, *argv);
        return 1;
    }

    /*
     * Read HV-matrix (by rows).  The input looks like this:
     *
     *     an1 an2 ... ann
     *
     *     ... ... ... ...
     *
     *     a21 a22 ... a2n
     *     a11 a12 ... a1n
     */
    for ( i = 0; i < nn; i++ )
        *(a + i) = read_unsigned_int();

    M(n_minus_one, n_minus_one) = A(n_minus_one, n_minus_one);
    /* For each cell, choose the less costly of two paths (moving from upper
       right corner down and to the left by diagonal layers) */
    for ( j = n - 2; j >= 0; j-- ) /* Above and including auxiliary diagonal */
        calc_diagonal_on_and_above(n_minus_one, j);
    for ( i = n - 2; i >= 0; i-- ) /* Below auxiliary diagonal */
        calc_diagonal_below(i, 0);

    /* Print out the result */
    printf("%u\n", M(0, 0));

    return 0;
}


/*
 * Calculates minimum cost of path to each of the matrix elements on diagonals
 * above auxiliary diagonal.  Diagonal starts at (i, j) as leftmost (uppermost)
 * element.
 */
void calc_diagonal_on_and_above(int i, int j)
{
    /* Uppermost row--take only path coming from right */
    M(i, j) = M_rt(i, j) + A(i, j);

    for ( i--, j++; i > 0 && j < n_minus_one; i--, j++ ) {
        M(i, j) = M_up(i, j);
        if ( M_rt(i, j) < M(i, j) )
            M(i, j) = M_rt(i, j);
        M(i, j) += A(i, j);
    }

    /* Rightmost column--take only path coming from up */
    M(i, j) = M_up(i, j) + A(i, j);

    return;
}


/*
 * Calculates minimum cost of path to each of the matrix elements on diagonals
 * below auxiliary diagonal.  Diagonal starts at (i, j) as leftmost (uppermost)
 * element.
 */
void calc_diagonal_below(int i, int j)
{
    for ( ; i >= 0 && j < n; i--, j++ ) {
        M(i, j) = M_up(i, j);
        if ( M_rt(i, j) < M(i, j) )
            M(i, j) = M_rt(i, j);
        M(i, j) += A(i, j);
    }

    return;
}


/*
 * Reads and returns an unsigned integer from stdin.  If an error occurs,
 * displays appropriated error message on stdout and exits the process with
 * return code 1.
 */
unsigned int read_unsigned_int(void)
{
    unsigned r;
    int t;

    t = scanf("%u", &r);
    if ( t == EOF ) {
        fprintf(stderr, errUnexpectedEOF, progname);
        exit(1);
    }
    if ( t == 0 ) {
        fprintf(stderr, errParsingInput, progname);
        exit(1);
    }

    return r;
}




/*
 * Local Variables:
 * compile-command: "gcc alt_climb.c && ./a.out << EOF
4
1 4 2 5
3 4 2 1
2 1 4 1
5 2 3 5
EOF"
 * End:
 */
