/*
 * Minimize complexity of product of n matrices by rearranging parentheses
 * (reordering multiplication), where complexity of product of n matrices is
 * defined as number of products of matrix elements.  Dimensions of matrices
 * are stored in array r of n+1 elements, so that dimensions of matricesf
 * M_1, M_2, ... , M_n are:
 *
 *   M_1(r_1, r_2), M_2(r_2, r_3), ... , M_n(r_n, r_{n+1})
 */
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>


char *errUnexpectedEOF = "%s: error--unexpected end of file\n";
char *errParsingInput  = "%s: error--invalid input\n";
char *errMemory        = "%s: error--cannot allocate enough memory\n";

char *progname;

unsigned  mcount; /* number of matrices */
unsigned *r;      /* array of matrix dimensions */
unsigned *m;      /* 2-D array for holding intermediate calculations */


#define COMPLEXITY(i, j)    *(m + (i) * mcount + j)
#define HEIGHT(i)           *(r + i)
#define WIDTH(i)            *(r + i + 1)

unsigned read_unsigned_int(void);


int main(int argc, char **argv)
{
    int i, j, k, l;
    unsigned min;

    progname = *argv;

    /* Read number of matrices */
    mcount = read_unsigned_int();

    /* Allocate buffer long enough to hold matrix sizes for all matrices */
    r = (unsigned *) malloc( sizeof(int) * ( mcount + 1 ) );
    /* Allocate 2-D buffer for (mcount * mcount) integers */
    m = (unsigned *) malloc( sizeof(int) * mcount * mcount );
    if ( !r || !m ) {
        fprintf(stderr, errMemory, *argv);
        return 1;
    }

    /* Read matrix sizes */
    for ( i = 0; i <= mcount; i++ )
        *(r + i) = read_unsigned_int();

    /*
     * Minimal complexity of multiplying group of matrices M_i, M_{i+1}, ... ,
     * M_j is stored in m[i][j].  Therefor:
     *
     *           { 0, i == j
     * m[i][j] = {
     *           { min_{i<=k<j} (m_{i,k} + m_{k+1,j} + r[i-1]*r[k]*r[j]), i < j
     *
     * We will have to repeatedly calculate product complexities for smaller
     * groups over and over when calculating complexities for larger groups.  To
     * prevent this redundancy, we start with smallest groups, saving their
     * complexities in 2-D array m, and later use them for calculation of
     * complexities for larger groups.
     */

    /* Product complexity for all groups of just one matrix (i == j) */
    for ( i = 0; i < mcount; i++ )
        *(m + mcount*i + i) = 0;

    /* Calculate complexities for groups of 2, 3, ... , mcount matrices */
    for ( l = 1; l < mcount; l++ )           /* current length of group - 1 */
        for ( i = 0; i < mcount - l; i++ ) { /* first element in the group */
            j = i + l;                       /* last element in the group */
            /*
             * Split current group (M_i, ... , M_j) into all possible pairs of
             * subgroups A and B, where
             *   A = M_i * ... * M_k,  B = M_{k+1} * ... * M_j,  k = i,...,j-1
             * For each possible split, calculate resulting complexity of
             * multiplying all matrices in current group.  Choose the minimal
             * complexity and store it in m[i][j] for later reference.
             *
             * Complexity of A * B is calculated as
             *      a * b * c,
             * where (a, b) and (b, c) are dimensions of respectively A and B.
             *
             * Complexities of products A and B have already been calculated and
             * saved in m[i][k] and m[k+1][j] during previous iterations.
             */
            COMPLEXITY(i, j) = UINT_MAX;     /* initial complexity ("infty") */
            for ( k = i; k < j; k++ ) {      /* last element of left subgroup */
                min = COMPLEXITY(i, k) + COMPLEXITY(k+1,j) +
                    HEIGHT(i) * WIDTH(k) * WIDTH(j);
                if ( COMPLEXITY(i, j) > min )
                    COMPLEXITY(i, j) = min;
            }
        }

    printf("%u\n", COMPLEXITY(0, mcount - 1));

    return 0;
}


/*
 * Reads and returns an unsigned integer from stdin.  If an error occurs,
 * displays appropriated error message on stdout and exits the process with
 * return code 1.
 */
unsigned read_unsigned_int(void)
{
    unsigned int r;
    int t;

    t = scanf("%u", &r);
    if ( t == EOF ) {
        fprintf(stderr, errUnexpectedEOF, progname);
        exit(1);
    }
    if ( t == 0 ) {
        fprintf(stderr, errParsingInput, progname);
        exit(1);
    }

    return r;
}




/*
 * Local Variables:
 * compile-command: "gcc matr_prod_complex.c && echo \"4 10 20 50 1 100\" | ./a.out"
 * End:
 */
